/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




// Created: Feb 15, 2000 by Andrew Caldwell from capoPlacer.h

#ifndef __CAPOPARAMS_H__
#define __CAPOPARAMS_H__

#include "MLPart/mlPart.h"
#include "SmallPlacers/baseSmallPl.h"

class CapoSplitterParams
{
  public:

        //if doRepartitioning, then the first tolerance is always
        //20%, and the second tolerance follows the parameters below
    bool        doRepartitioning;
 
        //if <useWSTolMethod>, then use the computation from the
	//whitespace report for horizontal cuts.  Otherwise,
	//use <constantTolerance>. Note that this is only for
	//h-cuts.  V-cuts always use a tolerance of 
	//<constantTolerance>, and shift the cutline.  
	//If doRepartitioning is enabled, then v-cuts will 
	//repartition with 1% tolerance after shifting.
    bool        useWSTolMethod;
    double      constantTolerance;
 
    unsigned    numMLSets;

    bool	useNameHierarchy;
    char*       delimiters;
 
    Verbosity   verb;
 
    CapoSplitterParams(int argc, const char *argv[]);
    CapoSplitterParams(Verbosity verbosity = Verbosity("0_0_0"))
        : doRepartitioning(true), useWSTolMethod(true),
          constantTolerance(10), numMLSets(2), 
	  useNameHierarchy(false), delimiters((char*)"/"), verb(verbosity)
        {}
 
   friend ostream& operator<<(ostream&, const CapoSplitterParams& params);
};
                   

enum SeedPlacerType {DumbPlacerSeed, BaryPlacerSeed, WeiszPlacerSeed};
enum ReplaceSmallBlocksType {Never, AtTheEnd, AtEveryLayer};        

class CapoSplitterParams;

class CapoParameters 
{
    void setDefaults();

  public: 
    Verbosity verb;

      //  PARAMETERS FOR TOP-DOWN FLOW
    unsigned stopAtBlocks;  //if !=0, stop when >= this many blocks
                            //default == 0 => run till end.

    ReplaceSmallBlocksType 	replaceSmallBlocks;

    bool     useActualPinLocs;

      //   PARAMETERS EQUALLY APPLICABLE TO EACH LEVEL
    unsigned smallPartThreshold; //for problems of size <= the
				//threshold, use smallPartitioner
    unsigned smallPlaceThreshold; //call the end-case placer on 
				//one-row problems with <= threshold #cells

    bool     doOverlapping;
    unsigned startOverlappingLayer;//the first level to overlap on
    unsigned endOverlappingLayer;  //the last level to overlap at

    bool	 propFromPl;	//do termi-prop w/ results of boxPl
    bool	 useKPl;


      //   PARTITIONER PARAMETERS
    MLPartParams	mlParams;
    SmallPlParams	smplParams;

      //lookAhead > 1 requires use of the LASplitter
    unsigned	lookAhead;

    bool		allowRowSplits;
    CapoSplitterParams  splitterParams;

      //  INFO-TRACKING PARAMETERS
    bool            plotBlocks;
    bool            saveLayerBBoxes;
    bool            saveBlocks;
    bool            saveSmallPlProb;

    CapoParameters();
    CapoParameters(int argc, const char *argv[]);
    CapoParameters(const CapoParameters &srcParams);

    void printHelp();

    friend ostream& operator<<(ostream& os, CapoParameters &params);
};


#endif
